{{ self_id }}
----TASK----
Answer the following user question:
--USER QUESTION--
{{ question }}
----
----RELATED CODEBASE CHANGES----
{% for part in diff %}{{ part }}\n{% endfor %}

----FULL FILE CONTENT AFTER APPLYING CHANGES----
{% for file, file_lines in all_file_lines.items() %}
--FILE: {{ file }}--
{{ file_lines }}
{% endfor %}

{% include "partial/aux_files.j2" %}

{%- if pipeline_out.associated_issue and pipeline_out.associated_issue.title %}
----ASSOCIATED ISSUE----
# {{ pipeline_out.associated_issue.title }}
{{ pipeline_out.associated_issue.description }}
URL: {{ pipeline_out.associated_issue.url }}
{%- endif -%}{{ '\n' }}

----ANSWERING INSTRUCTIONS----
{{ answering_instructions }}